@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,21 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -66,34 +62,53 @@ ContextPaneTextWidget::ContextPaneTextWidget(QWidget *parent) :
     ui->colorButton->setShowArrow(false);
     ui->textColorButton->setShowArrow(false);
 
-    connect(ui->colorButton, SIGNAL(toggled(bool)), this, SLOT(onColorButtonToggled(bool)));
-    connect(ui->textColorButton, SIGNAL(toggled(bool)), this, SLOT(onTextColorButtonToggled(bool)));
+    connect(ui->colorButton, &QmlEditorWidgets::ColorButton::toggled,
+            this, &ContextPaneTextWidget::onColorButtonToggled);
+    connect(ui->textColorButton, &QmlEditorWidgets::ColorButton::toggled,
+            this, &ContextPaneTextWidget::onTextColorButtonToggled);
 
     ContextPaneWidget *parentContextWidget = qobject_cast<ContextPaneWidget*>(parentWidget());
-    connect(parentContextWidget->colorDialog(), SIGNAL(accepted(QColor)), this, SLOT(onColorDialogApplied(QColor)));
-    connect(parentContextWidget->colorDialog(), SIGNAL(rejected()), this, SLOT(onColorDialogCancled()));
+    connect(parentContextWidget->colorDialog(), &CustomColorDialog::accepted,
+            this, &ContextPaneTextWidget::onColorDialogApplied);
+    connect(parentContextWidget->colorDialog(), &CustomColorDialog::rejected,
+            this, &ContextPaneTextWidget::onColorDialogCancled);
 
-    connect(ui->fontSizeSpinBox, SIGNAL(valueChanged(int)), this, SLOT(onFontSizeChanged(int)));
-    connect(ui->fontSizeSpinBox, SIGNAL(formatChanged()), this, SLOT(onFontFormatChanged()));
+    connect(ui->fontSizeSpinBox,
+            static_cast<void (QmlEditorWidgets::FontSizeSpinBox::*)(int)>(&QmlEditorWidgets::FontSizeSpinBox::valueChanged),
+            this, &ContextPaneTextWidget::onFontSizeChanged);
+    connect(ui->fontSizeSpinBox, &QmlEditorWidgets::FontSizeSpinBox::formatChanged,
+            this, &ContextPaneTextWidget::onFontFormatChanged);
 
-    connect(ui->boldButton, SIGNAL(toggled(bool)), this, SLOT(onBoldCheckedChanged(bool)));
-    connect(ui->italicButton, SIGNAL(toggled(bool)), this, SLOT(onItalicCheckedChanged(bool)));
-    connect(ui->underlineButton, SIGNAL(toggled(bool)), this, SLOT(onUnderlineCheckedChanged(bool)));
-    connect(ui->strikeoutButton, SIGNAL(toggled(bool)), this, SLOT(onStrikeoutCheckedChanged(bool)));
-    connect(ui->fontComboBox, SIGNAL(currentFontChanged(QFont)), this, SLOT(onCurrentFontChanged(QFont)));
+    connect(ui->boldButton, &QToolButton::toggled,
+            this, &ContextPaneTextWidget::onBoldCheckedChanged);
+    connect(ui->italicButton, &QToolButton::toggled,
+            this, &ContextPaneTextWidget::onItalicCheckedChanged);
+    connect(ui->underlineButton, &QToolButton::toggled,
+            this, &ContextPaneTextWidget::onUnderlineCheckedChanged);
+    connect(ui->strikeoutButton, &QToolButton::toggled,
+            this, &ContextPaneTextWidget::onStrikeoutCheckedChanged);
+    connect(ui->fontComboBox, &QFontComboBox::currentFontChanged,
+            this, &ContextPaneTextWidget::onCurrentFontChanged);
 
-    connect(ui->centerHAlignmentButton, SIGNAL(toggled(bool)), this, SLOT(onHorizontalAlignmentChanged()));
-    connect(ui->leftAlignmentButton, SIGNAL(toggled(bool)), this, SLOT(onHorizontalAlignmentChanged()));
-    connect(ui->rightAlignmentButton, SIGNAL(toggled(bool)), this, SLOT(onHorizontalAlignmentChanged()));
+    connect(ui->centerHAlignmentButton, &QToolButton::toggled,
+            this, &ContextPaneTextWidget::onHorizontalAlignmentChanged);
+    connect(ui->leftAlignmentButton, &QToolButton::toggled,
+            this, &ContextPaneTextWidget::onHorizontalAlignmentChanged);
+    connect(ui->rightAlignmentButton, &QToolButton::toggled,
+            this, &ContextPaneTextWidget::onHorizontalAlignmentChanged);
 
-    connect(ui->centerVAlignmentButton, SIGNAL(toggled(bool)), this, SLOT(onVerticalAlignmentChanged()));
-    connect(ui->topAlignmentButton, SIGNAL(toggled(bool)), this, SLOT(onVerticalAlignmentChanged()));
-    connect(ui->bottomAlignmentButton, SIGNAL(toggled(bool)), this, SLOT(onVerticalAlignmentChanged()));
+    connect(ui->centerVAlignmentButton, &QToolButton::toggled,
+            this, &ContextPaneTextWidget::onVerticalAlignmentChanged);
+    connect(ui->topAlignmentButton, &QToolButton::toggled,
+            this, &ContextPaneTextWidget::onVerticalAlignmentChanged);
+    connect(ui->bottomAlignmentButton, &QToolButton::toggled,
+            this, &ContextPaneTextWidget::onVerticalAlignmentChanged);
 
-    connect(ui->styleComboBox, SIGNAL(currentIndexChanged(QString)), this, SLOT(onStyleComboBoxChanged(QString)));
+    connect(ui->styleComboBox, static_cast<void (QComboBox::*)(const QString &)>(&QComboBox::currentIndexChanged),
+            this, &ContextPaneTextWidget::onStyleComboBoxChanged);
 }
 
-static inline bool checkIfBoolean(QVariant v)
+static inline bool checkIfBoolean(const QVariant &v)
 {
     return (v.toString() == QLatin1String("true") || v.toString() == QLatin1String("false"));
 }
