@@ -1,7 +1,7 @@
-/**************************************************************************
+/****************************************************************************
 **
-** Copyright (c) 2014 Brian McGillion and Hugues Delorme
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 Brian McGillion and Hugues Delorme
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,36 +9,35 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "vcsbaseclient.h"
-#include "command.h"
+#include "vcscommand.h"
 #include "vcsbaseclientsettings.h"
 #include "vcsbaseeditorparameterwidget.h"
 
 #include <coreplugin/icore.h>
 #include <coreplugin/vcsmanager.h>
+#include <coreplugin/editormanager/documentmodel.h>
+#include <coreplugin/editormanager/editormanager.h>
+#include <coreplugin/idocument.h>
 
 #include <utils/qtcassert.h>
 #include <utils/synchronousprocess.h>
 #include <vcsbase/vcsbaseeditor.h>
-#include <vcsbase/vcsbaseoutputwindow.h>
+#include <vcsbase/vcsoutputwindow.h>
 #include <vcsbase/vcsbaseplugin.h>
 
 #include <QStringList>
@@ -52,6 +51,17 @@
 #include <QVariant>
 #include <QProcessEnvironment>
 
+static void commandFinished(VcsBase::VcsBaseEditorWidget *editor, VcsBase::VcsCommand *cmd)
+{
+    if (!cmd->lastExecutionSuccess()) {
+        editor->reportCommandFinished(false, cmd->lastExecutionExitCode(), cmd->cookie());
+    } else if (cmd->cookie().type() == QVariant::Int) {
+        const int line = cmd->cookie().toInt();
+        if (line >= 0)
+            editor->gotoLine(line);
+    }
+}
+
 /*!
     \class VcsBase::VcsBaseClient
 
@@ -63,119 +73,236 @@
     \sa VcsBase::VcsJobRunner
 */
 
-Q_DECLARE_METATYPE(QVariant)
-
-inline Core::IEditor *locateEditor(const char *property, const QString &entry)
+static Core::IEditor *locateEditor(const char *property, const QString &entry)
 {
-    foreach (Core::IDocument *document, Core::EditorManager::documentModel()->openedDocuments())
+    foreach (Core::IDocument *document, Core::DocumentModel::openedDocuments())
         if (document->property(property).toString() == entry)
-            return Core::EditorManager::documentModel()->editorsForDocument(document).first();
+            return Core::DocumentModel::editorsForDocument(document).first();
     return 0;
 }
 
-namespace {
-
-VcsBase::VcsBaseOutputWindow *vcsOutputWindow()
-{
-    return VcsBase::VcsBaseOutputWindow::instance();
-}
-
-}
-
 namespace VcsBase {
 
+class VcsBaseClientImplPrivate
+{
+public:
+    VcsBaseClientImplPrivate(VcsBaseClientSettings *settings);
+    ~VcsBaseClientImplPrivate();
+
+    VcsBaseClientSettings *m_clientSettings;
+};
+
+VcsBaseClientImplPrivate::VcsBaseClientImplPrivate(VcsBaseClientSettings *settings) :
+    m_clientSettings(settings)
+{
+    m_clientSettings->readSettings(Core::ICore::settings());
+}
+
+VcsBaseClientImplPrivate::~VcsBaseClientImplPrivate()
+{
+    delete m_clientSettings;
+}
+
+VcsBaseClientImpl::VcsBaseClientImpl(VcsBaseClientSettings *settings) :
+    d(new VcsBaseClientImplPrivate(settings))
+{
+    connect(Core::ICore::instance(), &Core::ICore::saveSettingsRequested,
+            this, &VcsBaseClientImpl::saveSettings);
+}
+
+VcsBaseClientImpl::~VcsBaseClientImpl()
+{
+    delete d;
+}
+
+VcsBaseClientSettings &VcsBaseClientImpl::settings() const
+{
+    return *d->m_clientSettings;
+}
+
+Utils::FileName VcsBaseClientImpl::vcsBinary() const
+{
+    return settings().binaryPath();
+}
+
+VcsCommand *VcsBaseClientImpl::createCommand(const QString &workingDirectory,
+                                             VcsBaseEditorWidget *editor,
+                                             JobOutputBindMode mode) const
+{
+    auto cmd = new VcsCommand(workingDirectory, processEnvironment());
+    cmd->setDefaultTimeoutS(vcsTimeoutS());
+    if (editor) {
+        editor->setCommand(cmd);
+        connect(editor, &QObject::destroyed, cmd, &VcsCommand::abort);
+        connect(cmd, &VcsCommand::finished,
+                editor, [editor, cmd]() { commandFinished(editor, cmd); });
+    }
+    if (mode == VcsWindowOutputBind) {
+        cmd->addFlags(VcsCommand::ShowStdOut);
+        if (editor) // assume that the commands output is the important thing
+            cmd->addFlags(VcsCommand::SilentOutput);
+    } else if (editor) {
+        connect(cmd, &VcsCommand::stdOutText, editor, &VcsBaseEditorWidget::setPlainText);
+    }
+
+    return cmd;
+}
+
+void VcsBaseClientImpl::enqueueJob(VcsCommand *cmd, const QStringList &args,
+                                   const QString &workingDirectory,
+                                   Utils::ExitCodeInterpreter *interpreter)
+{
+    cmd->addJob(vcsBinary(), args, vcsTimeoutS(), workingDirectory, interpreter);
+    cmd->execute();
+}
+
+QProcessEnvironment VcsBaseClientImpl::processEnvironment() const
+{
+    QProcessEnvironment environment = QProcessEnvironment::systemEnvironment();
+    VcsBasePlugin::setProcessEnvironment(&environment, false);
+    return environment;
+}
+
+QString VcsBaseClientImpl::commandOutputFromLocal8Bit(const QByteArray &a)
+{
+    return Utils::SynchronousProcess::normalizeNewlines(QString::fromLocal8Bit(a));
+}
+
+QStringList VcsBaseClientImpl::commandOutputLinesFromLocal8Bit(const QByteArray &a)
+{
+    QString output = commandOutputFromLocal8Bit(a);
+    const QChar newLine = QLatin1Char('\n');
+    if (output.endsWith(newLine))
+        output.truncate(output.size() - 1);
+    if (output.isEmpty())
+        return QStringList();
+    return output.split(newLine);
+}
+
+void VcsBaseClientImpl::resetCachedVcsInfo(const QString &workingDir)
+{
+    Core::VcsManager::resetVersionControlForDirectory(workingDir);
+}
+
+void VcsBaseClientImpl::annotateRevisionRequested(const QString &workingDirectory,
+                                                  const QString &file, const QString &change,
+                                                  int line)
+{
+    QString changeCopy = change;
+    // This might be invoked with a verbose revision description
+    // "SHA1 author subject" from the annotation context menu. Strip the rest.
+    const int blankPos = changeCopy.indexOf(QLatin1Char(' '));
+    if (blankPos != -1)
+        changeCopy.truncate(blankPos);
+    annotate(workingDirectory, file, changeCopy, line);
+}
+
+bool VcsBaseClientImpl::vcsFullySynchronousExec(const QString &workingDir, const QStringList &args,
+                                                QByteArray *outputData, QByteArray *errorData,
+                                                unsigned flags) const
+{
+    QByteArray internalErrorData;
+    QScopedPointer<VcsCommand> command(createCommand(workingDir));
+    command->addFlags(flags);
+    bool result = command->runFullySynchronous(vcsBinary(), args, vcsTimeoutS(), outputData,
+                                               errorData ? errorData : &internalErrorData);
+    if (!internalErrorData.isEmpty() && !(flags & VcsCommand::SuppressStdErr))
+        VcsOutputWindow::appendError(commandOutputFromLocal8Bit(internalErrorData));
+    return result;
+}
+
+VcsCommand *VcsBaseClientImpl::vcsExec(const QString &workingDirectory, const QStringList &arguments,
+                                       VcsBaseEditorWidget *editor, bool useOutputToWindow,
+                                       unsigned additionalFlags, const QVariant &cookie)
+{
+    VcsCommand *command = createCommand(workingDirectory, editor,
+                                        useOutputToWindow ? VcsWindowOutputBind : NoOutputBind);
+    command->setCookie(cookie);
+    command->addFlags(additionalFlags);
+    if (editor)
+        command->setCodec(editor->codec());
+    enqueueJob(command, arguments);
+    return command;
+}
+
+Utils::SynchronousProcessResponse VcsBaseClientImpl::vcsSynchronousExec(const QString &workingDir,
+                                                                        const QStringList &args,
+                                                                        unsigned flags,
+                                                                        QTextCodec *outputCodec) const
+{
+    return VcsBasePlugin::runVcs(workingDir, vcsBinary(), args, vcsTimeoutS(), flags,
+                                 outputCodec, processEnvironment());
+}
+
+int VcsBaseClientImpl::vcsTimeoutS() const
+{
+    return settings().intValue(VcsBaseClientSettings::timeoutKey);
+}
+
+VcsBaseEditorWidget *VcsBaseClientImpl::createVcsEditor(Core::Id kind, QString title,
+                                                        const QString &source, QTextCodec *codec,
+                                                        const char *registerDynamicProperty,
+                                                        const QString &dynamicPropertyValue) const
+{
+    VcsBaseEditorWidget *baseEditor = 0;
+    Core::IEditor *outputEditor = locateEditor(registerDynamicProperty, dynamicPropertyValue);
+    const QString progressMsg = tr("Working...");
+    if (outputEditor) {
+        // Exists already
+        outputEditor->document()->setContents(progressMsg.toUtf8());
+        baseEditor = VcsBaseEditor::getVcsBaseEditor(outputEditor);
+        QTC_ASSERT(baseEditor, return 0);
+        Core::EditorManager::activateEditor(outputEditor);
+    } else {
+        outputEditor = Core::EditorManager::openEditorWithContents(kind, &title, progressMsg.toUtf8());
+        outputEditor->document()->setProperty(registerDynamicProperty, dynamicPropertyValue);
+        baseEditor = VcsBaseEditor::getVcsBaseEditor(outputEditor);
+        QTC_ASSERT(baseEditor, return 0);
+        connect(baseEditor, &VcsBaseEditorWidget::annotateRevisionRequested,
+                this, &VcsBaseClientImpl::annotateRevisionRequested);
+        baseEditor->setSource(source);
+        if (codec)
+            baseEditor->setCodec(codec);
+    }
+
+    baseEditor->setForceReadOnly(true);
+    return baseEditor;
+}
+
+void VcsBaseClientImpl::saveSettings()
+{
+    settings().writeSettings(Core::ICore::settings());
+}
+
 class VcsBaseClientPrivate
 {
 public:
-    VcsBaseClientPrivate(VcsBaseClient *client, VcsBaseClientSettings *settings);
+    VcsBaseEditorParameterWidget *createDiffEditor();
+    VcsBaseEditorParameterWidget *createLogEditor();
 
-    void statusParser(const QString &text);
-    void annotateRevision(const QString &workingDirectory, const QString &file,
-                          QString change, int lineNumber);
-    void saveSettings();
-
-    void bindCommandToEditor(Command *cmd, VcsBaseEditorWidget *editor);
-    void commandFinishedGotoLine(QWidget *editorObject);
-
-    VcsBaseClientSettings *m_clientSettings;
-    QSignalMapper *m_cmdFinishedMapper;
-
-private:
-    VcsBaseClient *m_client;
+    VcsBaseClient::ParameterWidgetCreator m_diffParamWidgetCreator;
+    VcsBaseClient::ParameterWidgetCreator m_logParamWidgetCreator;
 };
 
-VcsBaseClientPrivate::VcsBaseClientPrivate(VcsBaseClient *client, VcsBaseClientSettings *settings) :
-    m_clientSettings(settings),
-    m_cmdFinishedMapper(new QSignalMapper(client)),
-    m_client(client)
+VcsBaseEditorParameterWidget *VcsBaseClientPrivate::createDiffEditor()
 {
+    return m_diffParamWidgetCreator ? m_diffParamWidgetCreator() : 0;
 }
 
-void VcsBaseClientPrivate::statusParser(const QString &text)
+VcsBaseEditorParameterWidget *VcsBaseClientPrivate::createLogEditor()
 {
-    QList<VcsBaseClient::StatusItem> lineInfoList;
-
-    QStringList rawStatusList = text.split(QLatin1Char('\n'));
-
-    foreach (const QString &string, rawStatusList) {
-        const VcsBaseClient::StatusItem lineInfo = m_client->parseStatusLine(string);
-        if (!lineInfo.flags.isEmpty() && !lineInfo.file.isEmpty())
-            lineInfoList.append(lineInfo);
-    }
-
-    emit m_client->parsedStatus(lineInfoList);
-}
-
-void VcsBaseClientPrivate::annotateRevision(const QString &workingDirectory,  const QString &file,
-                                            QString change, int lineNumber)
-{
-    // This might be invoked with a verbose revision description
-    // "SHA1 author subject" from the annotation context menu. Strip the rest.
-    const int blankPos = change.indexOf(QLatin1Char(' '));
-    if (blankPos != -1)
-        change.truncate(blankPos);
-    m_client->annotate(workingDirectory, file, change, lineNumber);
-}
-
-void VcsBaseClientPrivate::saveSettings()
-{
-    m_clientSettings->writeSettings(Core::ICore::settings());
-}
-
-void VcsBaseClientPrivate::bindCommandToEditor(Command *cmd, VcsBaseEditorWidget *editor)
-{
-    editor->setCommand(cmd);
-    QObject::connect(cmd, SIGNAL(finished(bool,int,QVariant)), m_cmdFinishedMapper, SLOT(map()));
-    m_cmdFinishedMapper->setMapping(cmd, editor);
-}
-
-void VcsBaseClientPrivate::commandFinishedGotoLine(QWidget *editorObject)
-{
-    VcsBase::VcsBaseEditorWidget *editor = qobject_cast<VcsBase::VcsBaseEditorWidget *>(editorObject);
-    Command *cmd = qobject_cast<Command *>(m_cmdFinishedMapper->mapping(editor));
-    if (editor && cmd) {
-        if (!cmd->lastExecutionSuccess()) {
-            editor->reportCommandFinished(false, cmd->lastExecutionExitCode(), cmd->cookie());
-        } else if (cmd->cookie().type() == QVariant::Int) {
-            const int line = cmd->cookie().toInt();
-            if (line >= 0)
-                editor->gotoLine(line);
-        }
-        m_cmdFinishedMapper->removeMappings(cmd);
-    }
+    return m_logParamWidgetCreator ? m_logParamWidgetCreator() : 0;
 }
 
 VcsBaseClient::StatusItem::StatusItem(const QString &s, const QString &f) :
     flags(s), file(f)
-{
-}
+{ }
 
 VcsBaseClient::VcsBaseClient(VcsBaseClientSettings *settings) :
-    d(new VcsBaseClientPrivate(this, settings))
+    VcsBaseClientImpl(settings),
+    d(new VcsBaseClientPrivate)
 {
     qRegisterMetaType<QVariant>();
-    connect(Core::ICore::instance(), SIGNAL(saveSettingsRequested()), this, SLOT(saveSettings()));
-    connect(d->m_cmdFinishedMapper, SIGNAL(mapped(QWidget*)), this, SLOT(commandFinishedGotoLine(QWidget*)));
 }
 
 VcsBaseClient::~VcsBaseClient()
@@ -191,8 +318,7 @@ bool VcsBaseClient::synchronousCreateRepository(const QString &workingDirectory,
     QByteArray outputData;
     if (!vcsFullySynchronousExec(workingDirectory, args, &outputData))
         return false;
-    ::vcsOutputWindow()->append(
-                Utils::SynchronousProcess::normalizeNewlines(QString::fromLocal8Bit(outputData)));
+    VcsOutputWindow::append(commandOutputFromLocal8Bit(outputData));
 
     resetCachedVcsInfo(workingDirectory);
 
@@ -249,9 +375,9 @@ bool VcsBaseClient::synchronousPull(const QString &workingDir,
     args << vcsCommandString(PullCommand) << extraOptions << srcLocation;
     // Disable UNIX terminals to suppress SSH prompting
     const unsigned flags =
-            VcsBase::VcsBasePlugin::SshPasswordPrompt
-            | VcsBase::VcsBasePlugin::ShowStdOutInLogWindow
-            | VcsBase::VcsBasePlugin::ShowSuccessMessage;
+            VcsCommand::SshPasswordPrompt
+            | VcsCommand::ShowStdOut
+            | VcsCommand::ShowSuccessMessage;
     const Utils::SynchronousProcessResponse resp = vcsSynchronousExec(workingDir, args, flags);
     const bool ok = resp.result == Utils::SynchronousProcessResponse::Finished;
     if (ok)
@@ -267,65 +393,15 @@ bool VcsBaseClient::synchronousPush(const QString &workingDir,
     args << vcsCommandString(PushCommand) << extraOptions << dstLocation;
     // Disable UNIX terminals to suppress SSH prompting
     const unsigned flags =
-            VcsBase::VcsBasePlugin::SshPasswordPrompt
-            | VcsBase::VcsBasePlugin::ShowStdOutInLogWindow
-            | VcsBase::VcsBasePlugin::ShowSuccessMessage;
+            VcsCommand::SshPasswordPrompt
+            | VcsCommand::ShowStdOut
+            | VcsCommand::ShowSuccessMessage;
     const Utils::SynchronousProcessResponse resp = vcsSynchronousExec(workingDir, args, flags);
     return resp.result == Utils::SynchronousProcessResponse::Finished;
 }
 
-bool VcsBaseClient::vcsFullySynchronousExec(const QString &workingDir,
-                                            const QStringList &args,
-                                            QByteArray *output) const
-{
-    QProcess vcsProcess;
-    if (!workingDir.isEmpty())
-        vcsProcess.setWorkingDirectory(workingDir);
-    vcsProcess.setProcessEnvironment(processEnvironment());
-
-    const QString binary = settings()->binaryPath();
-
-    ::vcsOutputWindow()->appendCommand(workingDir, binary, args);
-
-    vcsProcess.start(binary, args);
-
-    if (!vcsProcess.waitForStarted()) {
-        ::vcsOutputWindow()->appendError(tr("Unable to start process '%1': %2")
-                                         .arg(QDir::toNativeSeparators(binary), vcsProcess.errorString()));
-        return false;
-    }
-
-    vcsProcess.closeWriteChannel();
-
-    QByteArray stdErr;
-    const int timeoutSec = settings()->intValue(VcsBaseClientSettings::timeoutKey);
-    if (!Utils::SynchronousProcess::readDataFromProcess(vcsProcess, timeoutSec * 1000,
-                                                        output, &stdErr, true)) {
-        Utils::SynchronousProcess::stopProcess(vcsProcess);
-        ::vcsOutputWindow()->appendError(tr("Timed out after %1s waiting for the process %2 to finish.")
-                                         .arg(timeoutSec).arg(binary));
-        return false;
-    }
-    if (!stdErr.isEmpty())
-        ::vcsOutputWindow()->appendError(QString::fromLocal8Bit(stdErr));
-
-    return vcsProcess.exitStatus() == QProcess::NormalExit && vcsProcess.exitCode() == 0;
-}
-
-Utils::SynchronousProcessResponse VcsBaseClient::vcsSynchronousExec(
-        const QString &workingDirectory,
-        const QStringList &args,
-        unsigned flags,
-        QTextCodec *outputCodec)
-{
-    const QString binary = settings()->binaryPath();
-    const int timeoutSec = settings()->intValue(VcsBaseClientSettings::timeoutKey);
-    return VcsBase::VcsBasePlugin::runVcs(workingDirectory, binary, args, timeoutSec * 1000,
-                                          flags, outputCodec);
-}
-
 void VcsBaseClient::annotate(const QString &workingDir, const QString &file,
-                             const QString revision /* = QString() */,
+                             const QString &revision /* = QString() */,
                              int lineNumber /* = -1 */,
                              const QStringList &extraOptions)
 {
@@ -333,14 +409,15 @@ void VcsBaseClient::annotate(const QString &workingDir, const QString &file,
     QStringList args;
     args << vcsCmdString << revisionSpec(revision) << extraOptions << file;
     const Core::Id kind = vcsEditorKind(AnnotateCommand);
-    const QString id = VcsBase::VcsBaseEditorWidget::getSource(workingDir, QStringList(file));
+    const QString id = VcsBaseEditor::getSource(workingDir, QStringList(file));
     const QString title = vcsEditorTitle(vcsCmdString, id);
-    const QString source = VcsBase::VcsBaseEditorWidget::getSource(workingDir, file);
+    const QString source = VcsBaseEditor::getSource(workingDir, file);
 
-    VcsBase::VcsBaseEditorWidget *editor = createVcsEditor(kind, title, source, true,
-                                                           vcsCmdString.toLatin1().constData(), id);
+    VcsBaseEditorWidget *editor = createVcsEditor(kind, title, source,
+                                                  VcsBaseEditor::getCodec(source),
+                                                  vcsCmdString.toLatin1().constData(), id);
 
-    Command *cmd = createCommand(workingDir, editor);
+    VcsCommand *cmd = createCommand(workingDir, editor);
     cmd->setCookie(lineNumber);
     enqueueJob(cmd, args);
 }
@@ -350,25 +427,31 @@ void VcsBaseClient::diff(const QString &workingDir, const QStringList &files,
 {
     const QString vcsCmdString = vcsCommandString(DiffCommand);
     const Core::Id kind = vcsEditorKind(DiffCommand);
-    const QString id = VcsBase::VcsBaseEditorWidget::getTitleId(workingDir, files);
+    const QString id = VcsBaseEditor::getTitleId(workingDir, files);
     const QString title = vcsEditorTitle(vcsCmdString, id);
-    const QString source = VcsBase::VcsBaseEditorWidget::getSource(workingDir, files);
-    VcsBase::VcsBaseEditorWidget *editor = createVcsEditor(kind, title, source, true,
-                                                           vcsCmdString.toLatin1().constData(), id);
+    const QString source = VcsBaseEditor::getSource(workingDir, files);
+    VcsBaseEditorWidget *editor = createVcsEditor(kind, title, source,
+                                                  VcsBaseEditor::getCodec(source),
+                                                  vcsCmdString.toLatin1().constData(), id);
     editor->setWorkingDirectory(workingDir);
 
     VcsBaseEditorParameterWidget *paramWidget = editor->configurationWidget();
-    if (!paramWidget && (paramWidget = createDiffEditor(workingDir, files, extraOptions))) {
+    if (!paramWidget && (paramWidget = d->createDiffEditor())) {
         // editor has been just created, createVcsEditor() didn't set a configuration widget yet
-        connect(editor, SIGNAL(diffChunkReverted(VcsBase::DiffChunk)), paramWidget, SLOT(executeCommand()));
+        connect(editor, &VcsBaseEditorWidget::diffChunkReverted,
+                paramWidget, &VcsBaseEditorParameterWidget::executeCommand);
+        connect(paramWidget, &VcsBaseEditorParameterWidget::commandExecutionRequested,
+                [=] { diff(workingDir, files, extraOptions); } );
         editor->setConfigurationWidget(paramWidget);
     }
 
     QStringList args;
     const QStringList paramArgs = paramWidget != 0 ? paramWidget->arguments() : QStringList();
     args << vcsCmdString << extraOptions << paramArgs << files;
-    Command *command = createCommand(workingDir, editor);
-    enqueueJob(command, args, exitCodeInterpreter(DiffCommand, command));
+    QTextCodec *codec = source.isEmpty() ? static_cast<QTextCodec *>(0) : VcsBaseEditor::getCodec(source);
+    VcsCommand *command = createCommand(workingDir, editor);
+    command->setCodec(codec);
+    enqueueJob(command, args, workingDir, exitCodeInterpreter(DiffCommand, command));
 }
 
 void VcsBaseClient::log(const QString &workingDir, const QStringList &files,
@@ -377,16 +460,19 @@ void VcsBaseClient::log(const QString &workingDir, const QStringList &files,
 {
     const QString vcsCmdString = vcsCommandString(LogCommand);
     const Core::Id kind = vcsEditorKind(LogCommand);
-    const QString id = VcsBase::VcsBaseEditorWidget::getTitleId(workingDir, files);
+    const QString id = VcsBaseEditor::getTitleId(workingDir, files);
     const QString title = vcsEditorTitle(vcsCmdString, id);
-    const QString source = VcsBase::VcsBaseEditorWidget::getSource(workingDir, files);
-    VcsBase::VcsBaseEditorWidget *editor = createVcsEditor(kind, title, source, true,
-                                                           vcsCmdString.toLatin1().constData(), id);
+    const QString source = VcsBaseEditor::getSource(workingDir, files);
+    VcsBaseEditorWidget *editor = createVcsEditor(kind, title, source,
+                                                  VcsBaseEditor::getCodec(source),
+                                                  vcsCmdString.toLatin1().constData(), id);
     editor->setFileLogAnnotateEnabled(enableAnnotationContextMenu);
 
     VcsBaseEditorParameterWidget *paramWidget = editor->configurationWidget();
-    if (!paramWidget && (paramWidget = createLogEditor(workingDir, files, extraOptions))) {
+    if (!paramWidget && (paramWidget = d->createLogEditor())) {
         // editor has been just created, createVcsEditor() didn't set a configuration widget yet
+        connect(paramWidget, &VcsBaseEditorParameterWidget::commandExecutionRequested,
+                [=]() { this->log(workingDir, files, extraOptions, enableAnnotationContextMenu); } );
         editor->setConfigurationWidget(paramWidget);
     }
 
@@ -404,9 +490,9 @@ void VcsBaseClient::revertFile(const QString &workingDir,
     QStringList args(vcsCommandString(RevertCommand));
     args << revisionSpec(revision) << extraOptions << file;
     // Indicate repository change or file list
-    Command *cmd = createCommand(workingDir);
+    VcsCommand *cmd = createCommand(workingDir);
     cmd->setCookie(QStringList(workingDir + QLatin1Char('/') + file));
-    connect(cmd, SIGNAL(success(QVariant)), this, SIGNAL(changed(QVariant)), Qt::QueuedConnection);
+    connect(cmd, &VcsCommand::success, this, &VcsBaseClient::changed, Qt::QueuedConnection);
     enqueueJob(cmd, args);
 }
 
@@ -416,9 +502,9 @@ void VcsBaseClient::revertAll(const QString &workingDir, const QString &revision
     QStringList args(vcsCommandString(RevertCommand));
     args << revisionSpec(revision) << extraOptions;
     // Indicate repository change or file list
-    Command *cmd = createCommand(workingDir);
+    VcsCommand *cmd = createCommand(workingDir);
     cmd->setCookie(QStringList(workingDir));
-    connect(cmd, SIGNAL(success(QVariant)), this, SIGNAL(changed(QVariant)), Qt::QueuedConnection);
+    connect(cmd, &VcsCommand::success, this, &VcsBaseClient::changed, Qt::QueuedConnection);
     enqueueJob(createCommand(workingDir), args);
 }
 
@@ -427,9 +513,10 @@ void VcsBaseClient::status(const QString &workingDir, const QString &file,
 {
     QStringList args(vcsCommandString(StatusCommand));
     args << extraOptions << file;
-    ::vcsOutputWindow()->setRepository(workingDir);
-    Command *cmd = createCommand(workingDir, 0, VcsWindowOutputBind);
-    connect(cmd, SIGNAL(finished(bool,int,QVariant)), ::vcsOutputWindow(), SLOT(clearRepository()),
+    VcsOutputWindow::setRepository(workingDir);
+    VcsCommand *cmd = createCommand(workingDir, 0, VcsWindowOutputBind);
+    connect(cmd, &VcsCommand::finished,
+            VcsOutputWindow::instance(), &VcsOutputWindow::clearRepository,
             Qt::QueuedConnection);
     enqueueJob(cmd, args);
 }
@@ -438,12 +525,12 @@ void VcsBaseClient::emitParsedStatus(const QString &repository, const QStringLis
 {
     QStringList args(vcsCommandString(StatusCommand));
     args << extraOptions;
-    Command *cmd = createCommand(repository);
-    connect(cmd, SIGNAL(output(QString)), this, SLOT(statusParser(QString)));
+    VcsCommand *cmd = createCommand(repository);
+    connect(cmd, &VcsCommand::stdOutText, this, &VcsBaseClient::statusParser);
     enqueueJob(cmd, args);
 }
 
-QString VcsBaseClient::vcsCommandString(VcsCommand cmd) const
+QString VcsBaseClient::vcsCommandString(VcsCommandTag cmd) const
 {
     switch (cmd) {
     case CreateRepositoryCommand: return QLatin1String("init");
@@ -465,13 +552,23 @@ QString VcsBaseClient::vcsCommandString(VcsCommand cmd) const
     return QString();
 }
 
-Utils::ExitCodeInterpreter *VcsBaseClient::exitCodeInterpreter(VcsCommand cmd, QObject *parent) const
+Utils::ExitCodeInterpreter *VcsBaseClient::exitCodeInterpreter(VcsCommandTag cmd, QObject *parent) const
 {
     Q_UNUSED(cmd)
     Q_UNUSED(parent)
     return 0;
 }
 
+void VcsBaseClient::setDiffParameterWidgetCreator(ParameterWidgetCreator creator)
+{
+    d->m_diffParamWidgetCreator = std::move(creator);
+}
+
+void VcsBaseClient::setLogParameterWidgetCreator(ParameterWidgetCreator creator)
+{
+    d->m_logParamWidgetCreator = std::move(creator);
+}
+
 void VcsBaseClient::import(const QString &repositoryRoot, const QStringList &files,
                            const QStringList &extraOptions)
 {
@@ -488,8 +585,8 @@ void VcsBaseClient::view(const QString &source, const QString &id,
     const Core::Id kind = vcsEditorKind(DiffCommand);
     const QString title = vcsEditorTitle(vcsCommandString(LogCommand), id);
 
-    VcsBase::VcsBaseEditorWidget *editor = createVcsEditor(kind, title, source,
-                                                           true, "view", id);
+    VcsBaseEditorWidget *editor = createVcsEditor(kind, title, source,
+                                                  VcsBaseEditor::getCodec(source), "view", id);
 
     const QFileInfo fi(source);
     const QString workingDirPath = fi.isFile() ? fi.absolutePath() : source;
@@ -501,9 +598,9 @@ void VcsBaseClient::update(const QString &repositoryRoot, const QString &revisio
 {
     QStringList args(vcsCommandString(UpdateCommand));
     args << revisionSpec(revision) << extraOptions;
-    Command *cmd = createCommand(repositoryRoot);
+    VcsCommand *cmd = createCommand(repositoryRoot);
     cmd->setCookie(repositoryRoot);
-    connect(cmd, SIGNAL(success(QVariant)), this, SIGNAL(changed(QVariant)), Qt::QueuedConnection);
+    connect(cmd, &VcsCommand::success, this, &VcsBaseClient::changed, Qt::QueuedConnection);
     enqueueJob(cmd, args);
 }
 
@@ -520,111 +617,34 @@ void VcsBaseClient::commit(const QString &repositoryRoot,
     //   So descendants of VcsBaseClient *must* redefine commit() and extend
     //   extraOptions with the usage for commitMessageFile (see BazaarClient::commit()
     //   for example)
-    Q_UNUSED(commitMessageFile);
     QStringList args(vcsCommandString(CommitCommand));
     args << extraOptions << files;
-    enqueueJob(createCommand(repositoryRoot), args);
-}
-
-VcsBaseClientSettings *VcsBaseClient::settings() const
-{
-    return d->m_clientSettings;
-}
-
-VcsBaseEditorParameterWidget *VcsBaseClient::createDiffEditor(const QString &workingDir,
-                                                              const QStringList &files,
-                                                              const QStringList &extraOptions)
-{
-    Q_UNUSED(workingDir);
-    Q_UNUSED(files);
-    Q_UNUSED(extraOptions);
-    return 0;
-}
-
-VcsBaseEditorParameterWidget *VcsBaseClient::createLogEditor(const QString &workingDir,
-                                                             const QStringList &files,
-                                                             const QStringList &extraOptions)
-{
-    Q_UNUSED(workingDir);
-    Q_UNUSED(files);
-    Q_UNUSED(extraOptions);
-    return 0;
+    VcsCommand *cmd = createCommand(repositoryRoot, 0, VcsWindowOutputBind);
+    if (!commitMessageFile.isEmpty())
+        connect(cmd, &VcsCommand::finished, [commitMessageFile]() { QFile(commitMessageFile).remove(); });
+    enqueueJob(cmd, args);
 }
 
 QString VcsBaseClient::vcsEditorTitle(const QString &vcsCmd, const QString &sourceId) const
 {
-    const QString binary = settings()->binaryPath();
-    return QFileInfo(binary).baseName() +
+    return vcsBinary().toFileInfo().baseName() +
             QLatin1Char(' ') + vcsCmd + QLatin1Char(' ') +
-            QFileInfo(sourceId).fileName();
+            Utils::FileName::fromString(sourceId).fileName();
 }
 
-VcsBase::VcsBaseEditorWidget *VcsBaseClient::createVcsEditor(Core::Id kind, QString title,
-                                                             const QString &source, bool setSourceCodec,
-                                                             const char *registerDynamicProperty,
-                                                             const QString &dynamicPropertyValue) const
+void VcsBaseClient::statusParser(const QString &text)
 {
-    VcsBase::VcsBaseEditorWidget *baseEditor = 0;
-    Core::IEditor *outputEditor = locateEditor(registerDynamicProperty, dynamicPropertyValue);
-    const QString progressMsg = tr("Working...");
-    if (outputEditor) {
-        // Exists already
-        outputEditor->document()->setContents(progressMsg.toUtf8());
-        baseEditor = VcsBase::VcsBaseEditorWidget::getVcsBaseEditor(outputEditor);
-        QTC_ASSERT(baseEditor, return 0);
-    } else {
-        outputEditor = Core::EditorManager::openEditorWithContents(kind, &title, progressMsg.toUtf8());
-        outputEditor->document()->setProperty(registerDynamicProperty, dynamicPropertyValue);
-        baseEditor = VcsBase::VcsBaseEditorWidget::getVcsBaseEditor(outputEditor);
-        connect(baseEditor, SIGNAL(annotateRevisionRequested(QString,QString,QString,int)),
-                this, SLOT(annotateRevision(QString,QString,QString,int)));
-        QTC_ASSERT(baseEditor, return 0);
-        baseEditor->setSource(source);
-        if (setSourceCodec)
-            baseEditor->setCodec(VcsBase::VcsBaseEditorWidget::getCodec(source));
+    QList<VcsBaseClient::StatusItem> lineInfoList;
+
+    QStringList rawStatusList = text.split(QLatin1Char('\n'));
+
+    foreach (const QString &string, rawStatusList) {
+        const VcsBaseClient::StatusItem lineInfo = parseStatusLine(string);
+        if (!lineInfo.flags.isEmpty() && !lineInfo.file.isEmpty())
+            lineInfoList.append(lineInfo);
     }
 
-    baseEditor->setForceReadOnly(true);
-    Core::EditorManager::activateEditor(outputEditor);
-    return baseEditor;
-}
-
-QProcessEnvironment VcsBaseClient::processEnvironment() const
-{
-    QProcessEnvironment environment = QProcessEnvironment::systemEnvironment();
-    VcsBase::VcsBasePlugin::setProcessEnvironment(&environment, false);
-    return environment;
-}
-
-Command *VcsBaseClient::createCommand(const QString &workingDirectory,
-                                      VcsBase::VcsBaseEditorWidget *editor,
-                                      JobOutputBindMode mode)
-{
-    Command *cmd = new Command(d->m_clientSettings->binaryPath(),
-                               workingDirectory, processEnvironment());
-    cmd->setDefaultTimeout(d->m_clientSettings->intValue(VcsBaseClientSettings::timeoutKey));
-    if (editor)
-        d->bindCommandToEditor(cmd, editor);
-    if (mode == VcsWindowOutputBind) {
-        cmd->addFlags(VcsBasePlugin::ShowStdOutInLogWindow);
-        if (editor) // assume that the commands output is the important thing
-            cmd->addFlags(VcsBasePlugin::SilentOutput);
-    } else if (editor) {
-        connect(cmd, SIGNAL(output(QString)), editor, SLOT(setPlainText(QString)));
-    }
-
-    return cmd;
-}
-
-void VcsBaseClient::enqueueJob(Command *cmd, const QStringList &args, Utils::ExitCodeInterpreter *interpreter)
-{
-    cmd->addJob(args, interpreter);
-    cmd->execute();
-}
-
-void VcsBaseClient::resetCachedVcsInfo(const QString &workingDir)
-{
-    Core::VcsManager::resetVersionControlForDirectory(workingDir);
+    emit parsedStatus(lineInfoList);
 }
 
 } // namespace VcsBase
